;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

;;; Mini-buffer

(defun write-mini-buffer (string)
  (clear-mini-buffer)
  (write-string string (rdoc-mini-buffer *application-frame*)))

(defun clear-mini-buffer ()
  (window-clear (rdoc-mini-buffer *application-frame*)))

(defun read-mini-buffer (&key default-string)
  (prog2
      (setf (rdoc-mini-buffer-active-p *application-frame*) t)
      (my-stream-read-line (rdoc-mini-buffer *application-frame*) 
			   :default-string default-string)
    (setf (rdoc-mini-buffer-active-p *application-frame*) nil)
    (clear-mini-buffer)))

(defmethod my-stream-read-line ((stream standard-extended-input-stream)
			&key default-string)
  (let ((result (make-array 1
			    :element-type 'character
			    :adjustable t
			    :fill-pointer 0)))
    (climi::with-encapsulating-stream (estream stream)     
      (when default-string
	(loop for c from 0 below (car (array-dimensions default-string))
	      do (progn
		   (let ((char (aref default-string c)))
		     (vector-push-extend char result)
		     (stream-write-char estream char)))))
      (loop for char = (climi::stream-read-char estream)
            while (not (char= char #\Newline))
            do (if (and (char= char #\Rubout) (not (equalp result #())))
		   (progn (vector-pop result)
			  (window-clear estream)
			  (stream-write-string estream result))
		   (progn (vector-push-extend char result)
			  (stream-write-char estream char)))
            finally (return result)))))
                      
