(in-package :RDOC)

;;; Arrays functions

(defmacro with-2d-array-1 ((width height) &body body)
  `(loop for r from 0 below ,height
	 do (loop for c from 0 below ,width
		  do ,@body)))

(defmacro with-2d-array ((array) &body body)
  (let ((width (gensym))
	(height (gensym)))	
    `(let ((,width (cadr (array-dimensions ,array)))
	   (,height (car (array-dimensions ,array))))
       (with-2d-array-1 (,width ,height) ,@body))))
  
(defun copy-array (src)
  (let* ((width (cadr (array-dimensions src)))
	 (height (car (array-dimensions src)))
	 (dest (make-array `(,height ,width) 
			   :element-type '(unsigned-byte 8))))
    (with-2d-array (src)
		   (setf (aref dest r c) (aref src r c)))
    dest))

(defun copy-sub-array-from-array (src width height origin-x origin-y)
  (let* ((width (floor width))
	 (height (floor height))
	 (origin-x (floor origin-x))
	 (origin-y (floor origin-y))
	 (dest (make-array `(,height ,width)
			   :element-type '(unsigned-byte 8))))
    (with-2d-array-1 (width height)
      (setf (aref dest r c) (aref src (+ origin-y r) (+ origin-x c))))
    dest))

(defun copy-sub-array-to-array (src dest origin-x origin-y)
  (let ((origin-x (floor origin-x))
	(origin-y (floor origin-y)))
    (with-2d-array (src)
      (setf (aref dest (+ origin-y r) (+ origin-x c))
	    (aref src r c)))))
    
;;; Make a GNUPlot histo plot

(defvar *gnuplot-filename* "gnuplot")

(defun make-gnuplot-histo (pixels filename)
  (let* ((script-file (format nil "~A.scr" filename))
	 (level-array (make-array '(256) :initial-element 0)))
    (when pixels
      (with-open-file (stream filename :direction :output)
	  (with-2d-array (pixels)
	    (incf (aref level-array (aref pixels r c))))
	(loop for c from 0 to 255
	      do (format stream "~A~%" (aref level-array c))))
      (with-open-file (stream script-file :direction :output)
	  (format stream "plot \"~A\" with lines~%" filename))
      (extensions:run-program *gnuplot-filename* 
			      (list "-persist" script-file)
			      :wait nil))))