;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

;; Zoom

(setf +pointer-right-button+ 3) ; clim-fix!

(define-tool (t :pointer-button-press +pointer-right-button+)
  (declare (ignore tool)) 
  (with-slots (center-zoom-x center-zoom-y) pane
    (multiple-value-bind (pos-x pos-y) 
	(untransform-position (medium-transformation (sheet-medium pane))
			      pos-x pos-y)
      (setf center-zoom-x pos-x
	    center-zoom-y pos-y)))
  (get-canvas-pixmap))

(defparameter zoom-factor 0.05)

(defun set-zoom-factor (new-factor)
  (setf zoom-factor new-factor))

(define-tool (t :pointer-motion +pointer-right-button+)
  (declare (ignore tool))
  (with-tool pane
    (let ((dif-y (- pos-y old-y)))
      (incf zoom (if (and (> dif-y 0) (<= zoom 1))
		     (- (* zoom-factor (log (+ 1 dif-y))))
		   (* zoom-factor (- dif-y)))))
    (when (< zoom 0.01)
      (setf zoom 0.01))
    (setf zoom (/ (round (* 100.0 zoom)) 100.0))
    (set-transformation pane)
    (put-canvas-pixmap)
    (draw-rectangle* pane
		     0 0 width height
		     :filled nil)))

(define-tool (t :pointer-button-release +pointer-right-button+)
  (declare (ignore tool pos-x pos-y))
  (dispatch-repaint pane nil))

;; Hand

(define-tool (:hand :pointer-button-press +pointer-middle-button+)
  (declare (ignore pos-x pos-y))
  (reset-canvas pane))

(define-tool (:hand :pointer-button-press +pointer-left-button+)
  (declare (ignore pos-x pos-y))
  (get-canvas-pixmap))

(define-tool (:hand :pointer-motion +pointer-left-button+)
  (with-tool pane
    (incf translate-x (- pos-x old-x))
    (incf translate-y (- pos-y old-y))
    (set-transformation pane)
    (put-canvas-pixmap)
    (draw-rectangle* pane
		     0 0 width height		    
		     :filled nil)))

(define-tool (:hand :pointer-button-release +pointer-left-button+)
  (declare (ignore pos-x pos-y))
  (dispatch-repaint pane nil))

;; Region tool

(defun my-make-rectangle* (x1 y1 x2 y2)
  (make-instance 'standard-rectangle :x1 x1 :y1 y1 :x2 x2 :y2 y2))

(defmacro my-draw-rectangle (sheet rectangle &rest rest)
  `(draw-rectangle* ,sheet 
                    (rectangle-min-x ,rectangle)
		    (rectangle-min-y ,rectangle)
		    (rectangle-max-x ,rectangle)
		    (rectangle-max-y ,rectangle)
		    ,@rest))

(defmacro crop-coord (x max)
  `(setf ,x (round (max 0 (min ,x ,max)))))

(define-tool (:region :pointer-button-press +pointer-left-button+) 
  (with-tool pane
    (with-slots (selected-region canvas-pixmap) pane
      (when selected-region
	(setf selected-region nil)
	(dispatch-repaint pane nil))
      (multiple-value-bind (tx ty) 
	  (untransform-position transform pos-x pos-y)
	(crop-it tx 0 width)
	(crop-it ty 0 height)
	(setf selected-region (my-make-rectangle* tx ty
						  tx ty)))
      (get-canvas-pixmap))))

(define-tool (:region :pointer-motion +pointer-left-button+)
  (with-tool pane
    (with-slots (selected-region canvas-pixmap) pane
      (multiple-value-bind (x1 y1 x2 y2) 
	  (rectangle-edges* selected-region)
	(declare (ignore x2 y2))
	  (multiple-value-bind (tx ty) 
	      (untransform-position transform pos-x pos-y)
	    (crop-it tx 0 width)
	    (crop-it ty 0 height)
	    (setf selected-region (my-make-rectangle* x1 y1 
						      tx ty)))
	  (put-canvas-pixmap)
	  (my-draw-rectangle pane selected-region
			     :filled nil)))))

(define-tool (:region :pointer-button-release +pointer-left-button+) 
  (with-tool pane
    (with-slots (selected-region canvas-pixmap) pane
      (multiple-value-bind (x1 y1 x2 y2) 
	  (rectangle-edges* selected-region)
	(if (and (= x1 x2) (= y1 y2))
	    (setf selected-region nil)
	    (progn 
	      (put-canvas-pixmap)
	      (crop-it x1 0 width)
	      (crop-it y1 0 height)
	      (crop-it x2 0 width)
	      (crop-it y2 0 height)
	      (setf selected-region 
		    (my-make-rectangle* (min x1 x2)
					(min y1 y2)
					(max x1 x2)
					(max y1 y2)))
	      (apply-tool-repaint pane tool nil)))))))

(defmethod apply-tool-repaint ((pane canvas-pane) 
			       (tool (eql :region)) region)
  (declare (ignore region))
  (with-slots (selected-region) pane
    (when selected-region
      (my-draw-rectangle pane selected-region 
			 :filled nil
			 :line-dashes t))))

;; Pen tool

(define-tool (:pen :pointer-button-press +pointer-left-button+)
  (with-tool pane 
    (when (>= tx 0)
      (setf (aref pic ty tx) (get-palette-color))
      (draw-rectangle* pane tx ty (1+ tx) (1+ ty)
		       :filled t
		       :ink (get-palette-color-object)))))

(define-tool (:pen :pointer-motion +pointer-left-button+) 
  (apply-tool pane tool :pointer-button-press button pos-x pos-y))
  
;; Eraser tool

(define-tool (:eraser :pointer-button-press +pointer-left-button+)
 (with-tool pane
    (when (>= tx 0)
      (setf (aref pic ty tx) 255)
      (draw-rectangle* pane tx ty (1+ tx) (1+ ty)
		       :filled t
		       :ink +white+))))

(define-tool (:eraser :pointer-motion +pointer-left-button+) 
  (apply-tool pane tool :pointer-button-press button pos-x pos-y))

;; Pipette tool

(define-tool (:pipette :pointer-button-press +pointer-left-button+) 
  (with-tool pane
    (when (>= tx 0)
      (set-palette-color (aref pic ty tx)))))

(define-tool (:pipette :pointer-motion +pointer-left-button+)
  (apply-tool pane tool :pointer-button-press button pos-x pos-y))
